// Particle class
// CSC 375 - Lea
// Jacob Peck - 20110913
// Assignment 1 - Threaded Particles


import java.util.*;
import java.awt.*;
import java.util.concurrent.*;
import java.util.concurrent.atomic.*;

public class Particle {
  // position
  private AtomicInteger x = new AtomicInteger();
  private AtomicInteger y = new AtomicInteger();
  // velocity
  private AtomicInteger deltax = new AtomicInteger();
  private AtomicInteger deltay = new AtomicInteger();
  // color
  private Color color;
  // size in pixels, width and height
  private AtomicInteger size = new AtomicInteger();
  
  
  // constructor
  public Particle(int x, int y, int deltax, int deltay, Color color, int size) {
    this.x.set(x);
    this.y.set(y);
    this.deltax.set(deltax);
    this.deltay.set(deltay);
    this.color = color;
    this.size.set(size);
  }
  
  // getters (threadsafe)
  public synchronized Point getCoordinates() {
    return new Point(x.get(),y.get());
  }
  
  public synchronized Point getVelocity() {
    return new Point(deltax.get(),deltay.get());
  }
  
  public synchronized Color getColor() {
    return color;
  }
  
  public int getSize() {
    return size.get();
  }
  
  // setters (threadsafe)
  public synchronized void setDeltaX(int newdeltax) {
    this.deltax.set(newdeltax);
  }
  
  public synchronized void setDeltaY(int newdeltay) {
    this.deltay.set(newdeltay);
  }
  
  public synchronized void setColor(Color newColor) {
    color = newColor;
  }
  
  
  // move method.  Moves by adding the delta values to x and y every tick,
  // checking for boundaries and collisions, and bouncing in either case
  public void move(int width, int height, CopyOnWriteArrayList<Particle> particles) {
    
    // standard move
    synchronized(this) {
      y.addAndGet(deltay.get());
      if (y.get() >= height-size.get() || y.get() <= 0) {
        if (y.get() <= 0) y.set(0);
        else y.set(height-size.get());
        deltay.set(deltay.get() * -1);
      }
      x.addAndGet(deltax.get());
      if (x.get() >= width-size.get() || x.get() <= 0) {
        if (x.get() <= 0) x.set(0);
        else x.set(width-size.get());
        deltax.set(deltax.get() * -1);
      }
    }
    
    // collision
    Rectangle thisRect = new Rectangle(x.get(), y.get(), size.get(), size.get());
    for (Particle p : particles) {
      if (p == this) continue;
      Point loc = p.getCoordinates();
      Point velo = p.getVelocity();
      int sz = p.getSize();
      Rectangle otherRect = new Rectangle(loc.getX(), loc.getY(), sz, sz);
      if (thisRect.intersects(otherRect)) {
        this.setDeltaY(this.deltay.get() * -1);
        this.setDeltaX(this.deltax.get() * -1);
        p.setDeltaY(velo.getY() * -1);
        p.setDeltaX(velo.getX() * -1);
      }
    }
  }
  
  
  // draw on the graphics
  public synchronized void draw(Graphics g) {
    Point point;
    int sz = size.get();
    Color drawColor;
    synchronized(this) {
      point = getCoordinates();
      drawColor = getColor();
    }
    g.setColor(drawColor);
    g.fillOval(point.getX(), point.getY(), sz, sz);  // circle
  }  
}
